# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ENVIRONMENT_VARIABLES = [
    'TRAVIS_BRANCH',
    'TRAVIS_BUILD_ID',
    'TRAVIS_BUILD_NUMBER',
    'TRAVIS_COMMIT',
    'TRAVIS_COMMIT_MESSAGE',
    'TRAVIS_COMMIT_RANGE',
    'TRAVIS_EVENT_TYPE',
    'TRAVIS_JOB_ID',
    'TRAVIS_JOB_NUMBER',
    'TRAVIS_OS_NAME',
    'TRAVIS_PULL_REQUEST',
    'TRAVIS_PULL_REQUEST_BRANCH',
    'TRAVIS_PULL_REQUEST_SHA',
    'TRAVIS_PULL_REQUEST_SLUG',
    'TRAVIS_REPO_SLUG',
    'TRAVIS_SUDO',
    'TRAVIS_TAG']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

import os
import platform

try:
    import _pytest._pluggy as pluggy
except ImportError:
    import pluggy
import pytest
import py

from pytest_metadata.ci import (
    appveyor, circleci, gitlab_ci, jenkins, taskcluster, travis_ci)


CONTINUOUS_INTEGRATION = {
    'AppVeyor': ['APPVEYOR', appveyor.ENVIRONMENT_VARIABLES],
    'CircleCI': ['CIRCLECI', circleci.ENVIRONMENT_VARIABLES],
    'GitLab CI': ['GITLAB_CI', gitlab_ci.ENVIRONMENT_VARIABLES],
    'Jenkins': ['JENKINS_URL', jenkins.ENVIRONMENT_VARIABLES],
    'TaskCluster': ['TASK_ID', taskcluster.ENVIRONMENT_VARIABLES],
    'Travis CI': ['TRAVIS', travis_ci.ENVIRONMENT_VARIABLES]}


@pytest.fixture(scope='session')
def metadata(pytestconfig):
    """Provide test session metadata"""
    return pytestconfig._metadata


def pytest_addoption(parser):
    parser.addoption('--metadata',
                     action='append',
                     default=[],
                     dest='metadata',
                     metavar=('key', 'value'),
                     nargs=2,
                     help='additional metadata.')


@pytest.hookimpl(tryfirst=True)
def pytest_configure(config):
    config._metadata = {
        'Python': platform.python_version(),
        'Platform': platform.platform(),
        'Packages': {
            'pytest': pytest.__version__,
            'py': py.__version__,
            'pluggy': pluggy.__version__}}
    config._metadata.update({
        k: v for k, v in config.getoption('metadata')})

    plugins = dict()
    for plugin, dist in config.pluginmanager.list_plugin_distinfo():
        name, version = dist.project_name, dist.version
        if name.startswith('pytest-'):
            name = name[7:]
        plugins[name] = version
    config._metadata['Plugins'] = plugins

    for key, value in CONTINUOUS_INTEGRATION.items():
        [config._metadata.update({v: os.environ.get(v)})
            for v in value[1] if os.environ.get(v)]

    if hasattr(config, 'slaveoutput'):
        config.slaveoutput['metadata'] = config._metadata


def pytest_report_header(config):
    if config.getoption('verbose') > 0:
        return 'metadata: {0}'.format(config._metadata)


@pytest.mark.optionalhook
def pytest_testnodedown(node):
    # note that any metadata from remote slaves will be replaced with the
    # environment from the final slave to quit
    if hasattr(node, 'slaveoutput'):
        node.config._metadata.update(node.slaveoutput['metadata'])



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ENVIRONMENT_VARIABLES = [
    'CI',
    'CI_COMMIT_REF_NAME',
    'CI_COMMIT_REF_SLUG',
    'CI_COMMIT_SHA',
    'CI_COMMIT_TAG',
    'CI_DEBUG_TRACE',
    'CI_ENVIRONMENT_NAME',
    'CI_ENVIRONMENT_SLUG',
    'CI_JOB_ID',
    'CI_JOB_MANUAL',
    'CI_JOB_NAME',
    'CI_JOB_STAGE',
    'CI_JOB_TOKEN',
    'CI_REPOSITORY_URL',
    'CI_RUNNER_DESCRIPTION',
    'CI_RUNNER_ID',
    'CI_RUNNER_TAGS',
    'CI_PIPELINE_ID',
    'CI_PIPELINE_TRIGGERED',
    'CI_PROJECT_DIR',
    'CI_PROJECT_ID',
    'CI_PROJECT_NAME',
    'CI_PROJECT_NAMESPACE',
    'CI_PROJECT_PATH',
    'CI_PROJECT_URL',
    'CI_REGISTRY',
    'CI_REGISTRY_IMAGE',
    'CI_REGISTRY_PASSWORD',
    'CI_REGISTRY_USER',
    'CI_SERVER',
    'CI_SERVER_NAME',
    'CI_SERVER_REVISION',
    'CI_SERVER_VERSION',
    'ARTIFACT_DOWNLOAD_ATTEMPTS',
    'GET_SOURCES_ATTEMPTS',
    'GITLAB_CI',
    'GITLAB_USER_ID',
    'GITLAB_USER_EMAIL',
    'RESTORE_CACHE_ATTEMPTS']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ENVIRONMENT_VARIABLES = [
    'CIRCLE_PROJECT_USERNAME',
    'CIRCLE_PROJECT_REPONAME',
    'CIRCLE_BRANCH',
    'CIRCLE_TAG',
    'CIRCLE_SHA1',
    'CIRCLE_REPOSITORY_URL',
    'CIRCLE_COMPARE_URL',
    'CIRCLE_BUILD_URL',
    'CIRCLE_BUILD_NUM',
    'CIRCLE_PREVIOUS_BUILD_NUM',
    'CI_PULL_REQUESTS',
    'CI_PULL_REQUEST',
    'CIRCLE_ARTIFACTS',
    'CIRCLE_USERNAME',
    'CIRCLE_TEST_REPORTS',
    'CIRCLE_PR_USERNAME',
    'CIRCLE_PR_REPONAME',
    'CIRCLE_PR_NUMBER',
    'CIRCLE_NODE_TOTAL',
    'CIRCLE_NODE_INDEX',
    'CIRCLE_BUILD_IMAGE']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ENVIRONMENT_VARIABLES = [
    'APPVEYOR_API_URL',
    'APPVEYOR_ACCOUNT_NAME',
    'APPVEYOR_PROJECT_ID',
    'APPVEYOR_PROJECT_NAME',
    'APPVEYOR_PROJECT_SLUG',
    'APPVEYOR_BUILD_FOLDER',
    'APPVEYOR_BUILD_ID',
    'APPVEYOR_BUILD_NUMBER',
    'APPVEYOR_BUILD_VERSION',
    'APPVEYOR_PULL_REQUEST_NUMBER',
    'APPVEYOR_PULL_REQUEST_TITLE',
    'APPVEYOR_JOB_ID',
    'APPVEYOR_JOB_NAME',
    'APPVEYOR_REPO_PROVIDER',
    'APPVEYOR_REPO_SCM',
    'APPVEYOR_REPO_NAME',
    'APPVEYOR_REPO_BRANCH',
    'APPVEYOR_REPO_TAG',
    'APPVEYOR_REPO_TAG_NAME',
    'APPVEYOR_REPO_COMMIT',
    'APPVEYOR_REPO_COMMIT_AUTHOR',
    'APPVEYOR_REPO_COMMIT_AUTHOR_EMAIL',
    'APPVEYOR_REPO_COMMIT_TIMESTAMP',
    'APPVEYOR_REPO_COMMIT_MESSAGE',
    'APPVEYOR_REPO_COMMIT_MESSAGE_EXTENDED',
    'APPVEYOR_SCHEDULED_BUILD',
    'APPVEYOR_FORCED_BUILD',
    'APPVEYOR_RE_BUILD',
    'PLATFORM',
    'CONFIGURATION']



# This Source Code Form is subject to the terms of the Mozilla Public
# License, v. 2.0. If a copy of the MPL was not distributed with this
# file, You can obtain one at http://mozilla.org/MPL/2.0/.

ENVIRONMENT_VARIABLES = [
    'BUILD_NUMBER',
    'BUILD_ID',
    'BUILD_URL',
    'NODE_NAME',
    'JOB_NAME',
    'BUILD_TAG',
    'EXECUTOR_NUMBER',
    'JENKINS_URL',
    'JAVA_HOME',
    'WORKSPACE',
    'SVN_REVISION',
    'CVS_BRANCH',
    'GIT_COMMIT',
    'GIT_URL',
    'GIT_BRANCH']
